@@ -76,9 +76,9 @@ locking is required.
 #define NEEDS_SWINSOCK
 #define PL_STREAM_IMPL
 #include "pl-stream.h"
-#include "SWI-Prolog.h"
 #include "pl-utf8.h"
 #include "../pl-mutex.h"
+#include "SWI-Prolog.h"
 #include <sys/types.h>
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -1907,6 +1907,11 @@ unallocStream(IOSTREAM *s)
   if ( s->context )
     Sdprintf("WARNING: unallocStream(): stream has context??\n");
 
+  if ( s->exception )
+  { PL_erase(s->exception);
+    s->exception = NULL;
+  }
+
   if ( !(s->flags & SIO_STATIC) )
     PL_free(s);
 }
@@ -4108,6 +4113,10 @@ Scleanup(void)
   for(i=0; i<=2; i++)
   { IOSTREAM *s = &S__iob[i];
 
+    if ( s->mbstate )
+      free(s->mbstate);
+    if ( s->locale )
+      releaseLocale(s->locale);
     s->bufp = s->buffer;		/* avoid actual flush */
     S__removebuf(s);
 
