@@ -112,6 +112,10 @@
  */
 #define PL_OPAQUE(type) type
 
+#ifdef __SANITIZE_ADDRESS__
+#include <sanitizer/lsan_interface.h>
+#endif
+
 #include "SWI-Prolog.h"
 
 /* Our definition of _PL_get_arg appears in pl-fli.h */
@@ -362,14 +366,14 @@ typedef _sigset_t sigset_t;
 #include <stdarg.h>
 #include <limits.h>
 
-#ifdef HAVE_SIGNAL_H 
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
 
 /* We have two important feature test macros relating to signals. The first is
  * O_SIGNALS, which controls whether Prolog itself should perform OS-level
  * signal handling, and is set by the USE_SIGNALS config directive.
- * 
+ *
  * The other macro is HAVE_OS_SIGNALS. This controls whether OS-level signal
  * handling is available at all, e.g. from the on_signal/3 predicate.
  */
@@ -449,7 +453,7 @@ typedef _sigset_t sigset_t;
 #if __STDC_VERSION__ >= 201112L && !defined(__STDC_NO_THREADS__) && !defined(__WINDOWS__)
 # include <threads.h>
 # undef thread_local /* we use this as an identifier, it's not in the C11 spec anyway */
-# define HAVE_STDC_THREADS 	1
+# define HAVE_STDC_THREADS	1
 #endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -611,11 +615,11 @@ sizes  of  the  hash  tables are defined.  Note that these should all be
 2^N.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define ATOMHASHSIZE		1024	/* global atom table */
+#define ATOMHASHSIZE		4096	/* global atom table */
 #define FUNCTORHASHSIZE		512	/* global functor table */
-#define PROCEDUREHASHSIZE	256	/* predicates in module user */
+#define PROCEDUREHASHSIZE	1024	/* predicates in module user */
 #define MODULEPROCEDUREHASHSIZE 16	/* predicates in other modules */
-#define MODULEHASHSIZE		16	/* global module table */
+#define MODULEHASHSIZE		128	/* global module table */
 #define PUBLICHASHSIZE		8	/* Module export table */
 #define FLAGHASHSIZE		16	/* global flag/3 table */
 
@@ -1220,19 +1224,6 @@ introduce a garbage collector (TBD).
 		 *	 INHIBIT SIGNALS	*
 		 *******************************/
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-At times an abort is not allowed because the heap  is  inconsistent  the
-programmer  should  call  startCritical  to start such a code region and
-endCritical to end it.
-
-MT/TBD: how to handle this gracefully in the multi-threading case.  Does
-it mean anything?
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-#define startCritical (void)(LD->critical++)
-#define endCritical   ((--(LD->critical) == 0 && LD->alerted) \
-				? f_endCritical() : TRUE)
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 LIST processing macros.
 
@@ -1621,6 +1612,9 @@ struct definition
   gen_t		last_modified;		/* Generation I was last modified */
   struct event_list  *events;		/* Forward update events */
   struct table_props *tabling;		/* Extended properties for tabling */
+#ifdef __SANITIZE_ADDRESS__
+  char	       *name;			/* Name for debugging */
+#endif
 #ifdef O_PROF_PENTIUM
   int		prof_index;		/* index in profiling */
   char	       *prof_name;		/* name in profiling */
@@ -2713,6 +2707,8 @@ typedef enum
 #define INFERENCE_NO_LIMIT 0x7fffffffffffffffLL /* Highest value */
 #endif
 
+#define CACHED_DICT_FUNCTORS 128	/* Max size of dict to cache */
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Administration of loaded intermediate code files  (see  pl-wic.c).  Used
 with the -c option to include all these if necessary.
