@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2012-2020, University of Amsterdam
+    Copyright (c)  2012-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -47,6 +48,7 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 #include "pl-prof.h"
 #include "pl-read.h"
 #include "pl-prims.h"
+#include "pl-comp.h"
 #include "pl-setup.h"
 #include "pl-fli.h"
 #include "pl-wic.h"
@@ -55,10 +57,13 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 #include "pl-proc.h"
 #include "pl-modul.h"
 #include "pl-flag.h"
+#include "pl-event.h"
 #include "pl-rec.h"
+#include "pl-tabling.h"
 #include "pl-term.h"
 #include "pl-funct.h"
 #include "pl-ext.h"
+#include "pl-cont.h"
 #include "pl-srcfile.h"
 #include "pl-load.h"
 #include "pl-nt.h"
@@ -508,11 +513,11 @@ initPaths(int argc, const char **argv)
 
 
 static void
-cleanupStringP(char **loc)
+setStringP(char **loc, const char *val)
 { char *s;
 
   if ( (s=*loc) )
-  { *loc = NULL;
+  { *loc = val ? store_string(val) : NULL;
     remove_string(s);
   }
 }
@@ -536,19 +541,19 @@ cleanupOptListP(opt_list **listp)
 
 static void
 cleanupPaths(void)
-{ cleanupStringP(&GD->paths.executable);
-  cleanupStringP(&systemDefaults.home);
-  cleanupStringP(&systemDefaults.startup);
-  cleanupStringP(&GD->options.systemInitFile);
-  cleanupStringP(&GD->options.compileOut);
-  cleanupStringP(&GD->options.topLevel);
-  cleanupStringP(&GD->options.initFile);
-  cleanupStringP(&GD->options.saveclass);
-  cleanupStringP(&GD->options.on_error);
-  cleanupStringP(&GD->options.on_warning);
-  cleanupStringP(&GD->os.myhome);
+{ setStringP(&GD->paths.executable, NULL);
+  setStringP(&systemDefaults.home, NULL);
+  setStringP(&systemDefaults.startup, NULL);
+  setStringP(&GD->options.systemInitFile, NULL);
+  setStringP(&GD->options.compileOut, NULL);
+  setStringP(&GD->options.topLevel, NULL);
+  setStringP(&GD->options.initFile, NULL);
+  setStringP(&GD->options.saveclass, NULL);
+  setStringP(&GD->options.on_error, NULL);
+  setStringP(&GD->options.on_warning, NULL);
+  setStringP(&GD->os.myhome, NULL);
 #ifdef __WINDOWS__
-  cleanupStringP(&GD->paths.module);
+  setStringP(&GD->paths.module, NULL);
 #endif
 
   cleanupOptListP(&GD->options.scriptFiles);
@@ -819,12 +824,12 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
       { GD->options.config = store_string(optval);
       } else if ( (optval=is_longopt(s, "on-error")) )
       { if ( on_error_style(optval) )
-	  GD->options.on_error = store_string(optval);
+	  setStringP(&GD->options.on_error, optval);
 	else
 	  return -1;
       } else if ( (optval=is_longopt(s, "on-warning")) )
       { if ( on_error_style(optval) )
-	  GD->options.on_warning = store_string(optval);
+	  setStringP(&GD->options.on_warning, optval);
 	else
 	  return -1;
       } else if ( !compile )
@@ -1134,6 +1139,7 @@ PL_initialise(int argc, char **argv)
     }
     argc = done;
     argv = argvleft;
+    GD->cmdline.appl_malloc = TRUE;
   } else
   { argc--;				/* saved state: only drop program */
     argv++;
@@ -1438,22 +1444,23 @@ run_on_halt(OnHalt *handlers, int rval)
 }
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cleanup Prolog. The reclaim_memory  argument   says  whether  the system
-tries to reclaim memory. This  is   true  when called from PL_cleanup().
-Ideally, this would allow for  restarting   the  system  without loosing
-memory. In practice, this is hard,   especially if foreign libraries are
-loaded.
+Cleanup Prolog.  `status` carries the exit status as well as flags.  See
+the docs for PL_cleanup() for a description.
 
-When called from PL_halt(),  reclaim_memory   is  FALSE, unless compiled
-with -DGC_DEBUG.
+When called from PL_halt(), PL_CLEANUP_NO_RECLAIM_MEMORY  is set, unless
+compiled for debugging or using ASAN.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define MAX_HALT_CANCELLED 10
 
 int
-cleanupProlog(int rval, int reclaim_memory)
-{ if ( GD->cleaning != CLN_NORMAL )
-    return FALSE;
+PL_cleanup(int status)
+{ int rval = status&PL_CLEANUP_STATUS_MASK;
+  int asked_reclaim_memory = (status&PL_CLEANUP_NO_RECLAIM_MEMORY) == 0;
+  int reclaim_memory = asked_reclaim_memory;
+
+  if ( GD->cleaning != CLN_NORMAL )
+    return PL_CLEANUP_RECURSIVE;
 
 #ifdef __WINDOWS__
   if ( rval != 0 && !hasConsole() )
@@ -1463,7 +1470,7 @@ cleanupProlog(int rval, int reclaim_memory)
   PL_LOCK(L_INIT);
   if ( GD->cleaning != CLN_NORMAL )
   { PL_UNLOCK(L_INIT);
-    return FALSE;
+    return PL_CLEANUP_RECURSIVE;
   }
 
 #ifdef O_PLMT
@@ -1488,20 +1495,21 @@ cleanupProlog(int rval, int reclaim_memory)
 
     PL_set_prolog_flag("exit_status", PL_INTEGER, rval);
     if ( query_loop(PL_new_atom("$run_at_halt"), FALSE) == FALSE &&
-	 rval == 0 )
+	 !(status&PL_CLEANUP_NO_CANCEL) )
     { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
       { GD->cleaning = CLN_NORMAL;
 	PL_UNLOCK(L_INIT);
-	return FALSE;
+	return PL_CLEANUP_CANCELED;
       }
     }
 
     GD->cleaning = CLN_FOREIGN;
-    if ( !run_on_halt(&GD->os.on_halt_list, rval) && rval == 0 )
+    if ( !run_on_halt(&GD->os.on_halt_list, rval) &&
+	 !(status&PL_CLEANUP_NO_CANCEL) )
     { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
       { GD->cleaning = CLN_NORMAL;
 	PL_UNLOCK(L_INIT);
-	return FALSE;
+	return PL_CLEANUP_CANCELED;
       }
     }
   }
@@ -1513,7 +1521,12 @@ cleanupProlog(int rval, int reclaim_memory)
 #endif
 #ifdef O_PLMT
   if ( !exitPrologThreads() )		/* reclaim memory while a thread */
-    reclaim_memory = FALSE;		/* runs is likely to crash */
+  { if ( reclaim_memory )
+    { Sdprintf("WARNING: Failed to stop Prolog threads. "
+	       "Not reclaming memory.\n");
+      reclaim_memory = FALSE;
+    }
+  }
 
 emergency:
 #endif
@@ -1540,33 +1553,31 @@ emergency:
 
   RemoveTemporaryFiles();
 
-  if ( GD->resources.DB )
-  { zip_close_archive(GD->resources.DB);
-    GD->resources.DB = NULL;
-  }
-
   cleanupSignals();
-#ifdef HAVE_DMALLOC_H
-  dmalloc_verify(0);
-#endif
 
   if ( reclaim_memory )
-  { freeStacks();
+  { cleanupOs();
+    freeStacks();
+    cleanupBreakPoints();
 #ifdef O_PLMT
     cleanupLocalDefinitions(LD);
 #endif
-    freePrologLocalData(LD);
     cleanupModules();
+    cleanupProcedures();
     cleanupPrologFlags();
     cleanupFlags();
     cleanupRecords();
     cleanupTerm();
+    freePrologLocalData(LD);
+    cleanupEvents();
+    cleanupTabling();
     cleanupAtoms();
     cleanupFunctors();
     cleanupArith();
     cleanupInitialiseHooks();
     cleanupExtensions();
-    cleanupOs();
+    cleanupWamTable();
+    cleanupLocale();
     cleanupSourceFiles();
     Scleanup();
 #ifdef O_PLMT
@@ -1575,26 +1586,48 @@ emergency:
     cleanupForeign();
     cleanupPaths();
     cleanupCodeToAtom();
+    cleanupCont();
 #ifdef O_GMP
     cleanupGMP();
 #endif
     cleanupDebug();
+
+    if ( GD->cmdline.appl_malloc )
+      free(GD->cmdline.appl_argv);
+  }
+
+  if ( GD->resources.DB )
+  { zip_close_archive(GD->resources.DB);
+    GD->resources.DB = NULL;
   }
 
   PL_UNLOCK(L_INIT);				/* requires GD->thread.enabled */
 
+  DEBUG(0, assert(GD->clauses.lingering == 0));
+
   if ( reclaim_memory )
   { memset(&PL_global_data, 0, sizeof(PL_global_data));
     memset(&PL_local_data,  0, sizeof(PL_local_data));
   }
 
-  return TRUE;
-}
+#ifdef __SANITIZE_ADDRESS__
+  char *s;
 
+  if ( (s=getenv("ASAN_OPTIONS")) && strstr(s,"detect_leaks=1") )
+  { fprintf(stderr, "Running LSAN memory leak check (reclaim_memory=%d)\n",
+	   reclaim_memory);
+    if ( __lsan_do_recoverable_leak_check() )
+    { fprintf(stderr, "Leaks detected; sleeping 60 sec.  Attach using\n"
+	     "   gdb -p %d\n", getpid());
+      sleep(60);
+    } else
+    { fprintf(stderr, "No leaks detected\n");
+    }
+  }
+#endif
 
-int
-PL_cleanup(int rc)
-{ return cleanupProlog(rc, rc == 0);
+  return reclaim_memory == asked_reclaim_memory ? PL_CLEANUP_SUCCESS
+						: PL_CLEANUP_FAILED;
 }
 
 
