@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2020, University of Amsterdam
+    Copyright (c)  1985-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -47,6 +48,7 @@
 #include "pl-trie.h"
 #include "pl-tabling.h"
 #include "pl-undo.h"
+#include "pl-event.h"
 #include "pl-fli.h"
 #include "pl-funct.h"
 #include "pl-modul.h"
@@ -108,7 +110,6 @@ setupProlog(void)
     LD->signal.pending[i] = 0;
   LD->statistics.start_time = WallTime();
 
-  startCritical;
   DEBUG(1, Sdprintf("wam_table ...\n"));
   initWamTable();
   DEBUG(1, Sdprintf("character types ...\n"));
@@ -172,9 +173,6 @@ setupProlog(void)
   GD->clauses.cgc_stack_factor  = 0.03;
   GD->clauses.cgc_clause_factor = 1.0;
 
-  if ( !endCritical )
-    return FALSE;
-
   DEBUG(1, Sdprintf("Heap Initialised\n"));
   return TRUE;
 }
@@ -865,6 +863,8 @@ static void
 alert_handler(int sig)
 { SigHandler sh = &GD->signals.handlers[SIGNAL_INDEX(sig)];
 
+  DEBUG(MSG_THREAD_SIGNAL, Sdprintf("[%d]: received alert\n", PL_thread_self()));
+
   if ( sh->saved_handler &&
        sh->saved_handler != SIG_IGN &&
        sh->saved_handler != SIG_DFL )
@@ -1204,7 +1204,11 @@ handleSignals(DECL_LD)
     return done;
 #endif
   if ( LD->critical )
+  { DEBUG(MSG_THREAD_SIGNAL,
+	  Sdprintf("[%d]: ignoring signal (critical = %d)\n",
+		   PL_thread_self(), LD->critical));
     return 0;
+  }
 
   for(i=0; i<SIGMASK_WORDS; i++)
   { while( LD->signal.pending[i] )
@@ -1271,9 +1275,15 @@ PRED_IMPL("prolog_alert_signal", 2, prolog_alert_signal, 0)
 #endif
 
 
+void
+startCritical(DECL_LD)
+{ LD->critical++;
+}
+
+
 int
-f_endCritical(DECL_LD)
-{ if ( exception_term )
+endCritical(DECL_LD)
+{ if ( --LD->critical == 0 && LD->alerted && exception_term )
     return FALSE;
 
   return TRUE;
@@ -1749,19 +1759,36 @@ freePrologLocalData(PL_local_data_t *ld)
     destroyHTable(ld->prolog_flag.table);
     PL_UNLOCK(L_PLFLAG);
   }
+  free_predicate_references(ld);
+  destroy_event_list(&ld->event.hook.onthreadexit);
+  free_thread_wait(ld);
+#endif
+
+#ifdef O_LOCALE
+  if ( ld->locale.current )
+    releaseLocale(ld->locale.current);
 #endif
 
   if ( ld->qlf.getstr_buffer )
     free(ld->qlf.getstr_buffer);
-  if ( ld->tabling.node_pool )
-    free_alloc_pool(ld->tabling.node_pool);
 
   clearThreadTablingData(ld);
+  if ( ld->tabling.node_pool )
+    free_alloc_pool(ld->tabling.node_pool);
 
 #ifdef O_C_STACK_GUARDED
   if ( ld->signal.alt_stack )
     free(ld->signal.alt_stack);
 #endif
+
+  free_undo_data(ld);
+
+  if ( ld->btrace_store )
+  { btrace_destroy(ld->btrace_store);
+    ld->btrace_store = NULL;
+  }
+
+  cleanAbortHooks(ld);
 }
 
 /* The following definitions aren't necessary for compiling, and in fact
