@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2018, University of Amsterdam
-                         VU University Amsterdam
-		         CWI, Amsterdam
+    Copyright (c)  2018-2022, University of Amsterdam
+			      VU University Amsterdam
+			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -37,7 +38,24 @@
 #define __MINGW_USE_VC2005_COMPAT		/* Get Windows time_t as 64-bit */
 
 #include "pl-incl.h"
+
+#if defined(HAVE_MMAP) || defined(__WINDOWS__)
+#define HAVE_FILE_MAPPING 1
+
+typedef struct mapped_file
+{ char *start;
+  char *end;
+#ifdef __WINDOWS__
+  HANDLE hfile;					/* handle to the file */
+  HANDLE hmap;					/* handle to the map */
+#endif
+} mapped_file;
+
+static void	unmap_file(mapped_file *mf);
+#endif
+
 #include "pl-zip.h"
+#include "pl-fli.h"
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -476,6 +494,12 @@ close_zipper(zipper *z)
     }
     z->input.any = NULL;
   }
+#ifdef HAVE_FILE_MAPPING
+  if ( z->mapped_file )
+  { unmap_file(z->mapped_file);
+    z->mapped_file = NULL;
+  }
+#endif
 
   return rc;
 }
@@ -484,7 +508,10 @@ static int
 release_zipper(atom_t aref)
 { zipper *z = PL_blob_data(aref, NULL, NULL);
 
+  if ( z == GD->resources.DB )
+    GD->resources.DB = NULL;
   close_zipper(z);
+
 #ifdef O_PLMT
   simpleMutexDelete(&z->lock);
 #endif
@@ -825,6 +852,7 @@ static const opt_spec zip_new_file_options[] =
   { ATOM_time,		    OPT_DOUBLE },
   { ATOM_method,	    OPT_ATOM   },
   { ATOM_level,		    OPT_INT    },
+  { ATOM_zip64,		    OPT_BOOL   },
   { NULL_ATOM,		    0          }
 };
 
@@ -842,9 +870,10 @@ PRED_IMPL("zipper_open_new_file_in_zip", 4, zipper_open_new_file_in_zip, 0)
   int level = 6;
   atom_t method = ATOM_deflated;
   int imethod;
+  int zip64 = FALSE;
 
   if ( !scan_options(A4, 0, ATOM_zip_options, zip_new_file_options,
-		     &extra, &comment, &ftime, &method, &level) )
+		     &extra, &comment, &ftime, &method, &level, &zip64) )
     return FALSE;
 
   if ( extra )
@@ -891,7 +920,7 @@ PRED_IMPL("zipper_open_new_file_in_zip", 4, zipper_open_new_file_in_zip, 0)
 				 0,		/* crc */
 				 VERSIONMADEBY,	/* versionMadeBy */
 				 0,		/* flagBase */
-				 FALSE);	/* zip64 */
+				 zip64);	/* zip64 */
 
     if ( rc == 0 )
     { IOSTREAM *s = Snew(z, SIO_OUTPUT, &Szipfunctions);
@@ -1113,17 +1142,7 @@ PRED_IMPL("$rc_handle", 1, rc_handle, 0)
 		 *	      MAPPING		*
 		 *******************************/
 
-#if defined(HAVE_MMAP) || defined(__WINDOWS__)
-#define HAVE_FILE_MAPPING 1
-
-typedef struct mapped_file
-{ char *start;
-  char *end;
-#ifdef __WINDOWS__
-  HANDLE hfile;					/* handle to the file */
-  HANDLE hmap;					/* handle to the map */
-#endif
-} mapped_file;
+#ifdef HAVE_FILE_MAPPING
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void *)-1)
@@ -1241,9 +1260,6 @@ unmap_file(mapped_file *mf)
 		 *	 ARCHIVE EMULATION	*
 		 *******************************/
 
-/* TBD: get rid of the mapped_file on close
-*/
-
 zipper *
 zip_open_archive(const char *file, int flags)
 { zipper z = {0};
@@ -1257,9 +1273,12 @@ zip_open_archive(const char *file, int flags)
     DEBUG(MSG_ZIP, Sdprintf("Opening %s using file mapping\n", file));
 
     if ( (mf=map_file(file)) )
-    { if ( !(r=zip_open_archive_mem((const unsigned char *)mf->start,
+    { if ( (r=zip_open_archive_mem((const unsigned char *)mf->start,
 				    mf->end-mf->start, flags)) )
-	unmap_file(mf);
+      { r->mapped_file = mf;
+      } else
+      { unmap_file(mf);
+      }
     }
 #else
     DEBUG(MSG_ZIP, Sdprintf("Opening %s as stream\n", file));
@@ -1272,8 +1291,8 @@ zip_open_archive(const char *file, int flags)
 #ifdef O_PLMT
       simpleMutexInit(&r->lock);
 #endif
-      r->input_type = ZIP_FILE;
-      r->input.any  = NULL;
+      r->input_type  = ZIP_FILE;
+      r->input.any   = NULL;
     }
 #endif
   } else
@@ -1329,7 +1348,10 @@ zip_open_archive_mem(const unsigned char *mem, size_t mem_size, int flags)
 
 int
 zip_close_archive(zipper *z)
-{ return close_zipper(z);
+{ int rc = close_zipper(z);
+
+  free(z);
+  return rc;
 }
 
 
